#!/bin/bash

MONITOR_UNIX_PATH=""
UART_UNIX_PATH=""
RAM_SIZE=""
RAM_FILE=""
SIMPLE=0

while [[ "$#" -gt 0 ]]; do
    case $1 in
        --monitor_unix_path) MONITOR_UNIX_PATH="$2"; shift ;;
        --uart_unix_path) UART_UNIX_PATH="$2"; shift ;;
        --ram_file) RAM_FILE="$2"; shift ;;
        --ram_size) RAM_SIZE=$(($2 / 1024 / 1024)); shift ;;
        --simple) SIMPLE=1;;
        *) echo "Unknown parameter: $1" >&2; exit 1 ;;
    esac
    shift
done

if [[ ${SIMPLE} = 1 ]]; then
    MONITOR_UNIX_PATH="/tmp/monitor.unix"
    UART_UNIX_PATH="/tmp/uart.unix"
    RAM_FILE=/tmp/memory.ram
    RAM_SIZE=256
fi

if [[ -z ${MONITOR_UNIX_PATH} ]]; then
    echo "Monitor path not provided"
    exit 1
fi

if [[ -z ${UART_UNIX_PATH} ]]; then
    echo "Uart path not provided"
    exit 1
fi

if [[ -z ${RAM_SIZE} ]]; then
    echo "RAM size not provided"
    exit 1
fi

if [[ -z ${RAM_FILE} ]]; then
    echo "RAM size not provided"
    exit 1
fi

Args=(
    -nographic
    -display none
    -vga none
    -net none
    -no-reboot
    -snapshot
    -fw_cfg name="opt/org.tianocore/X-Cpuhp-Bugcheck-Override",string=yes
    -machine q35
    -drive if=pflash,format=raw,unit=0,file=OVMF_CODE.fd,readonly=on
    -drive if=pflash,format=raw,unit=1,file=OVMF_VARS.fd
    -drive file=disk_image.img,format=raw

    -fw_cfg name=opt/flag,file=flag.txt

    # Memory setup
    # This is necessary to allow aliasing the RAM into two physical locations while still treating the
    # memory as separate QEMU memory regions.
    -object memory-backend-file,size=${RAM_SIZE}m,id=memory.ram,mem-path=${RAM_FILE},share=on,prealloc=on
    -m ${RAM_SIZE}m
    -M memory-backend=memory.ram

    -cpu "hxp-hack2"

    # The monitor is used for reading / writing memory
    -monitor unix:${MONITOR_UNIX_PATH},server,nowait

    # Makes it uniform for handling IO with the frontend app
    -serial unix:${UART_UNIX_PATH},server,nowait

    # Enable debugging
    # -s
)

./qemu_system_x86_64_sev "${Args[@]}"
